using System;
using System.Text;
using System.Data;
using System.Collections.Generic;
using System.Collections;
using System.Web;

using CMS.IO;
using CMS.SettingsProvider;
using CMS.GlobalHelper;
using CMS.TranslationServices;

namespace SampleTranslationServices
{
    /// <summary>
    /// Sample translations provider.
    /// </summary>
    public class SampleTranslationService : AbstractHumanTranslationService
    {
        #region "Properties"

        /// <summary>
        /// Folder path where translation submission are exported.
        /// </summary>
        public string ExportFolder
        {
            get
            {
                string folder = SettingsKeyProvider.GetStringValue(SiteName + ".CMSManualTranslationExportFolder");
                if (string.IsNullOrEmpty(folder))
                {
                    folder = "~/App_Data/Translations/Export/";
                }
                return URLHelper.GetPhysicalPath(folder);
            }
        }


        /// <summary>
        /// Folder path where the translated submission are retrieved from.
        /// </summary>
        public string ImportFolder
        {
            get
            {
                string folder = SettingsKeyProvider.GetStringValue(SiteName + ".CMSManualTranslationImportFolder");
                if (string.IsNullOrEmpty(folder))
                {
                    folder = "~/App_Data/Translations/Import/";
                }
                return URLHelper.GetPhysicalPath(folder);
            }
        }

        #endregion


        #region "Public methods"

        /// <summary>
        /// Checks if everything required to run the service is in the settings of the service.
        /// </summary>
        public override bool IsAvailable()
        {
            // This service needs only paths, but if nothing is in settings, default values are used
            return true;
        }


        /// <summary>
        /// Checks if language is supported within the service
        /// </summary>
        /// <param name="langCode">Code of the culture</param>
        public override bool IsLanguageSupported(string langCode)
        {
            // All languages are supported
            return true;
        }


        /// <summary>
        /// Creates new submission (or resubmits existing if submission ticket is present).
        /// </summary>
        /// <param name="submission">Submission object</param>
        public override string CreateSubmission(TranslationSubmissionInfo submission)
        {
            try
            {
                if (submission != null)
                {
                    // Get the unique path to the file
                    string path = null;
                    if (string.IsNullOrEmpty(submission.SubmissionTicket))
                    {
                        path = Path.Combine(this.ExportFolder, ValidationHelper.GetSafeFileName(submission.SubmissionName) + ".zip");
                        path = FileHelper.GetUniqueFileName(path);

                        // Submission ticket is zip file name
                        submission.SubmissionTicket = Path.GetFileName(path);
                    }
                    else
                    {
                        // Resubmit action - uses the same path, overwrites the file if exists
                        path = Path.Combine(this.ExportFolder, submission.SubmissionTicket);
                    }

                    // Write the file
                    DirectoryHelper.EnsureDiskPath(path, null);
                    using (FileStream stream = File.Create(path))
                    {
                        TranslationServiceHelper.WriteSubmissionInZIP(submission, stream);
                    }
                }
            }
            catch (Exception ex)
            {
                TranslationServiceHelper.LogEvent(ex);
                return ex.Message;
            }
            return null;
        }


        /// <summary>
        /// Cancels given submission.
        /// </summary>
        /// <param name="submission">Submission to cancel</param>
        public override string CancelSubmission(TranslationSubmissionInfo submission)
        {
            try
            {
                if (submission != null)
                {
                    // Try to delete the zip file (path is saved as the submission ticket)
                    string path = Path.Combine(this.ExportFolder, submission.SubmissionTicket);
                    if (File.Exists(path))
                    {
                        File.Delete(path);
                    }
                }
            }
            catch (Exception ex)
            {
                TranslationServiceHelper.LogEvent(ex);
                return ex.Message;
            }
            return null;
        }


        /// <summary>
        /// Retrieves completed XLIFF files from the service and processes them (imports them into the system). Returns true if everything went well.
        /// </summary>
        public override string DownloadCompletedTranslations()
        {
            try
            {
                if (Directory.Exists(this.ImportFolder))
                {
                    string[] files = Directory.GetFiles(this.ImportFolder, "*.zip");
                    foreach (string filePath in files)
                    {
                        string file = Path.GetFileName(filePath);

                        // Get the submissions which belogs to this zip file (the path is stored as a SubmmissionTicket).
                        DataSet ds = TranslationSubmissionInfoProvider.GetSubmissions("SubmissionTicket = '" + SqlHelperClass.GetSafeQueryString(file, false) + "'", null);
                        if (!DataHelper.DataSourceIsEmpty(ds))
                        {
                            foreach (DataRow dr in ds.Tables[0].Rows)
                            {
                                TranslationSubmissionInfo submission = new TranslationSubmissionInfo(dr);

                                // Translate only Waiting for translation
                                if (submission.SubmissionStatus == TranslationStatusEnum.WaitingForTranslation)
                                {
                                    // Get the zip name
                                    string fileName = submission.SubmissionTicket;
                                    string path = Path.Combine(this.ImportFolder, fileName);
                                    if (File.Exists(path))
                                    {
                                        // Import xliff files from zip
                                        string err = TranslationServiceHelper.ImportXLIFFfromZIP(submission, FileStream.New(path, FileMode.Open));
                                        if (string.IsNullOrEmpty(err))
                                        {
                                            // Change the status to Translation ready and save the submission
                                            submission.SubmissionStatus = TranslationStatusEnum.TranslationReady;
                                            TranslationSubmissionInfoProvider.SetTranslationSubmissionInfo(submission);
                                        }
                                        else
                                        {
                                            return err;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                return null;
            }
            catch (Exception ex)
            {
                TranslationServiceHelper.LogEvent(ex);
                return ex.Message;
            }
        }

        #endregion
    }
}